@@ -1,5 +1,5 @@
 use crate::cli::error::CliError;
-use crate::cli::utils::{maybe_plural, write_file, Config, FileProcessing};
+use crate::cli::utils::{log_array, maybe_plural, write_file, Config, FileProcessing};
 use crate::cli::GlobalOptions;
 
 use darklua_core::{
@@ -26,9 +26,7 @@ pub struct Options {
 
 type MinifyResult = Result<(), CliError>;
 
-fn process(file: &FileProcessing, options: &Options, global: &GlobalOptions) -> MinifyResult {
-    let config = Config::new(&options.config_path, global)?;
-
+fn minify(file: &FileProcessing, config: &Config) -> MinifyResult {
     let source = &file.source;
     let output = &file.output;
 
@@ -52,19 +50,36 @@ fn process(file: &FileProcessing, options: &Options, global: &GlobalOptions) ->
     write_file(output, &minified)
         .map_err(|io_error| CliError::OutputFile(output.clone(), format!("{}", io_error)))?;
 
-    if global.verbose > 0 {
-        println!("Successfully processed <{}>", source.to_string_lossy());
-    };
+    log::debug!("Successfully processed <{}>", source.display());
 
     Ok(())
 }
 
 pub fn run(options: &Options, global: &GlobalOptions) {
-    let file = FileProcessing::find(&options.input_path, &options.output_path, global);
+    log::debug!("running `minify`: {:?}", options);
 
-    let results: Vec<MinifyResult> = file
+    let files = FileProcessing::find(&options.input_path, &options.output_path, global);
+
+    log::trace!(
+        "planned work: {}",
+        log_array(files.iter().map(|file| if file.is_in_place() {
+            format!("{}", file.source.display())
+        } else {
+            format!("{} -> {}", file.source.display(), file.output.display())
+        }))
+    );
+
+    let config = match Config::new(&options.config_path) {
+        Ok(config) => config,
+        Err(error) => {
+            eprintln!("{}", error);
+            return;
+        }
+    };
+
+    let results: Vec<MinifyResult> = files
         .iter()
-        .map(|file_processing| process(file_processing, options, global))
+        .map(|file_processing| minify(file_processing, &config))
         .collect();
 
     let total_files = results.len();
@@ -80,7 +95,7 @@ pub fn run(options: &Options, global: &GlobalOptions) {
     let error_count = errors.len();
 
     if error_count == 0 {
-        println!(
+        log::info!(
             "Successfully minified {} file{}",
             total_files,
             maybe_plural(total_files)
@@ -92,16 +107,21 @@ pub fn run(options: &Options, global: &GlobalOptions) {
             eprintln!(
                 "Successfully minified {} file{}.",
                 success_count,
-                maybe_plural(success_count)
+                maybe_plural(success_count),
+            );
+            eprintln!(
+                "But {} error{} happened:",
+                error_count,
+                maybe_plural(error_count)
+            );
+        } else {
+            eprintln!(
+                "{} error{} happened:",
+                error_count,
+                maybe_plural(error_count)
             );
         }
 
-        eprintln!(
-            "But {} error{} happened:",
-            error_count,
-            maybe_plural(error_count)
-        );
-
         errors.iter().for_each(|error| eprintln!("-> {}", error));
         process::exit(1);
     }
