@@ -1,6 +1,8 @@
 mod config;
+mod timer;
 
 pub use config::*;
+pub use timer::*;
 
 use crate::cli::GlobalOptions;
 
@@ -34,6 +36,10 @@ impl FileProcessing {
             Vec::new()
         }
     }
+
+    pub fn is_in_place(&self) -> bool {
+        self.source == self.output
+    }
 }
 
 fn walk_dir(path: &Path, output: &Path, files: &mut Vec<FileProcessing>, global: &GlobalOptions) {
@@ -65,14 +71,14 @@ fn walk_dir(path: &Path, output: &Path, files: &mut Vec<FileProcessing>, global:
                         output: file_output,
                     });
                 }
-            } else if global.verbose > 1 {
-                println!(
+            } else {
+                log::info!(
                     "Unexpected directory entry: {}",
                     file_path.to_string_lossy()
                 );
             }
-        } else if global.verbose > 1 {
-            println!("No file name for path {}", file_path.to_string_lossy());
+        } else {
+            log::info!("No file name for path {}", file_path.to_string_lossy());
         }
     }
 }
@@ -96,3 +102,13 @@ pub fn maybe_plural(count: usize) -> &'static str {
         ""
     }
 }
+
+pub fn log_array(iterator: impl Iterator<Item = String>) -> String {
+    let mut elements: Vec<_> = iterator.collect();
+    if elements.len() > 7 {
+        let difference = elements.len() - 7;
+        elements.truncate(7);
+        elements.push(format!("... and {} more", difference));
+    }
+    format!("[\n    {}\n]", elements.join(",\n    "))
+}
