@@ -3,10 +3,12 @@
 
 mod dense;
 mod readable;
+mod token_based;
 mod utils;
 
 pub use dense::DenseLuaGenerator;
 pub use readable::ReadableLuaGenerator;
+pub use token_based::TokenBasedLuaGenerator;
 
 use crate::nodes;
 
@@ -50,20 +52,51 @@ pub trait LuaGenerator {
     fn write_repeat_statement(&mut self, repeat: &nodes::RepeatStatement);
     fn write_while_statement(&mut self, while_statement: &nodes::WhileStatement);
 
+    fn write_variable(&mut self, variable: &nodes::Variable) {
+        use nodes::Variable::*;
+        match variable {
+            Identifier(identifier) => self.write_identifier(identifier),
+            Field(field) => self.write_field(field),
+            Index(index) => self.write_index(index),
+        }
+    }
+
     fn write_expression(&mut self, expression: &nodes::Expression);
 
+    fn write_identifier(&mut self, identifier: &nodes::Identifier);
     fn write_binary_expression(&mut self, binary: &nodes::BinaryExpression);
     fn write_unary_expression(&mut self, unary: &nodes::UnaryExpression);
     fn write_function(&mut self, function: &nodes::FunctionExpression);
     fn write_function_call(&mut self, call: &nodes::FunctionCall);
     fn write_field(&mut self, field: &nodes::FieldExpression);
     fn write_index(&mut self, index: &nodes::IndexExpression);
-    fn write_prefix(&mut self, prefix: &nodes::Prefix);
+    fn write_parenthese(&mut self, parenthese: &nodes::ParentheseExpression);
+
+    fn write_prefix(&mut self, prefix: &nodes::Prefix) {
+        use nodes::Prefix::*;
+        match prefix {
+            Call(call) => self.write_function_call(call),
+            Field(field) => self.write_field(field),
+            Identifier(identifier) => self.write_identifier(identifier),
+            Index(index) => self.write_index(index),
+            Parenthese(parenthese) => self.write_parenthese(parenthese),
+        }
+    }
+
     fn write_table(&mut self, table: &nodes::TableExpression);
     fn write_table_entry(&mut self, entry: &nodes::TableEntry);
     fn write_number(&mut self, number: &nodes::NumberExpression);
 
-    fn write_arguments(&mut self, arguments: &nodes::Arguments);
+    fn write_arguments(&mut self, arguments: &nodes::Arguments) {
+        use nodes::Arguments::*;
+        match arguments {
+            String(string) => self.write_string(string),
+            Table(table) => self.write_table(table),
+            Tuple(tuple) => self.write_tuple_arguments(tuple),
+        }
+    }
+
+    fn write_tuple_arguments(&mut self, arguments: &nodes::TupleArguments);
 
     fn write_string(&mut self, string: &nodes::StringExpression);
 }
@@ -129,6 +162,33 @@ mod test {
         };
     }
 
+    macro_rules! blocks_consistency {
+        (
+            $generator:expr => (
+                $($name:ident => $code:literal),+,
+            )
+        ) => {
+            $(
+                #[test]
+                fn $name() {
+                    let parser = $crate::Parser::default();
+
+                    let expected_block = parser.parse($code)
+                        .expect(&format!("unable to parse `{}`", $code));
+
+                    let mut generator = $generator;
+                    generator.write_block(&expected_block);
+                    let generated_code = generator.into_string();
+
+                    let generated_block = parser.parse(&generated_code)
+                        .expect(&format!("unable to parse generated code `{}`", &generated_code));
+
+                    assert_eq!(expected_block, generated_block);
+                }
+            )*
+        };
+    }
+
     macro_rules! binary_precedence {
         (
             $generator:expr => (
@@ -146,25 +206,28 @@ mod test {
                         .expect("it should have a return statement");
 
                     let expected = match expected_return {
-                        LastStatement::Return(expressions) => expressions.first().unwrap(),
+                        LastStatement::Return(statement) => statement.iter_expressions()
+                            .next()
+                            .unwrap(),
                         _ => panic!("return statement expected"),
                     };
 
                     let mut generator = $generator;
                     generator.write_expression(&$input.into());
 
-                    let parsed_block = parser.parse(&format!("return {}", generator.into_string()))
-                        .unwrap();
+                    let generated_code = format!("return {}", generator.into_string());
+                    let parsed_block = parser.parse(&generated_code)
+                        .expect(&format!("unable to parse generated code: `{}`", &generated_code));
 
                     let parsed_return = parsed_block.get_last_statement()
                         .expect("it should have a return statement");
 
                     let parsed = match parsed_return {
-                        LastStatement::Return(expressions) => {
-                            if expressions.len() != 1 {
+                        LastStatement::Return(statement) => {
+                            if statement.len() != 1 {
                                 panic!("return statement has more than one expression")
                             }
-                            expressions.first().unwrap()
+                            statement.iter_expressions().next().unwrap()
                         },
                         _ => panic!("return statement expected"),
                     };
@@ -182,6 +245,20 @@ mod $mod_name {
     use super::*;
     use $crate::nodes::*;
 
+    mod edge_cases {
+        use super::*;
+
+        blocks_consistency!($generator => (
+            index_with_bracket_string => "return ok[ [[field]]]",
+            call_with_bracket_string => "return ok[[ [field] ]]",
+            concat_numbers => "return 9 .. 3",
+            concat_number_with_variable_arguments => "return 9 .. ...",
+            concat_variable_arguments_with_number => "return ... ..1",
+            double_unary_minus => "return - -10",
+            binary_minus_with_unary_minus => "return 100- -10",
+        ));
+    }
+
     mod numbers {
         use super::*;
 
@@ -213,18 +290,18 @@ mod $mod_name {
             left_associative_wraps_right_operand_if_has_lower_precedence(
                 BinaryExpression::new(
                     BinaryOperator::And,
-                    Expression::False,
+                    false,
                     BinaryExpression::new(
                         BinaryOperator::Or,
-                        Expression::False,
-                        Expression::True,
+                        false,
+                        true,
                     ),
                 )
             ) => "false and (false or true)",
             left_associative_wraps_right_operand_if_has_same_precedence(
                 BinaryExpression::new(
                     BinaryOperator::Equal,
-                    Expression::True,
+                    true,
                     BinaryExpression::new(
                         BinaryOperator::LowerThan,
                         DecimalNumber::new(1.0),
@@ -302,25 +379,25 @@ mod $mod_name {
     mod snapshots {
         use super::*;
         snapshot_node!($mod_name, $generator, expression, write_expression => (
-            false_value => Expression::False,
-            true_value => Expression::True,
-            nil_value => Expression::Nil,
-            variable_arguments => Expression::VariableArguments,
-            true_in_parenthese => Expression::Parenthese(Box::new(Expression::True)),
+            false_value => false,
+            true_value => true,
+            nil_value => Expression::nil(),
+            variable_arguments => Expression::variable_arguments(),
+            true_in_parenthese => Expression::from(true).in_parentheses(),
         ));
 
         snapshot_node!($mod_name, $generator, assign, write_statement => (
             variable_with_one_value => AssignStatement::new(
                 vec![Variable::new("var")],
-                vec![Expression::False],
+                vec![Expression::from(false)],
             ),
             two_variables_with_one_value => AssignStatement::new(
                 vec![Variable::new("foo"), Variable::new("var")],
-                vec![Expression::False],
+                vec![Expression::from(false)],
             ),
             two_variables_with_two_values => AssignStatement::new(
                 vec![Variable::new("foo"), Variable::new("var")],
-                vec![Expression::Nil, Expression::False],
+                vec![Expression::nil(), Expression::from(false)],
             ),
         ));
 
@@ -342,7 +419,7 @@ mod $mod_name {
         snapshot_node!($mod_name, $generator, function_statement, write_statement => (
             empty => FunctionStatement::from_name("foo", Block::default()),
             empty_with_field =>  FunctionStatement::new(
-                FunctionName::from_name("foo").with_fields(vec!["bar".to_owned()]),
+                FunctionName::from_name("foo").with_fields(vec!["bar".into()]),
                 Block::default(),
                 Vec::new(),
                 false
@@ -357,19 +434,19 @@ mod $mod_name {
 
         snapshot_node!($mod_name, $generator, generic_for, write_statement => (
             empty => GenericForStatement::new(
-                vec!["var".to_owned()],
-                vec![Expression::True],
+                vec!["var".into()],
+                vec![Expression::from(true)],
                 Block::default()
             ),
         ));
 
         snapshot_node!($mod_name, $generator, if_statement, write_statement => (
-            empty => IfStatement::create(Expression::False, Block::default()),
-            empty_with_empty_else => IfStatement::create(Expression::False, Block::default())
+            empty => IfStatement::create(false, Block::default()),
+            empty_with_empty_else => IfStatement::create(false, Block::default())
                 .with_else_block(Block::default()),
-            empty_with_empty_multiple_branch => IfStatement::create(Expression::False, Block::default())
-                .with_branch(Expression::Nil, Block::default())
-                .with_branch(Expression::False, Block::default()),
+            empty_with_empty_multiple_branch => IfStatement::create(false, Block::default())
+                .with_new_branch(Expression::nil(), Block::default())
+                .with_new_branch(false, Block::default()),
         ));
 
         snapshot_node!($mod_name, $generator, local_assign, write_statement => (
@@ -377,7 +454,7 @@ mod $mod_name {
             foo_and_bar_unassigned => LocalAssignStatement::from_variable("foo")
                 .with_variable("bar"),
             var_assign_to_false => LocalAssignStatement::from_variable("var")
-                .with_value(Expression::False),
+                .with_value(false),
         ));
 
         snapshot_node!($mod_name, $generator, local_function, write_statement => (
@@ -397,16 +474,16 @@ mod $mod_name {
         snapshot_node!($mod_name, $generator, numeric_for, write_statement => (
             empty_without_step => NumericForStatement::new(
                 "i",
-                Expression::Identifier("start".to_owned()),
-                Expression::Identifier("max".to_owned()),
+                Expression::identifier("start"),
+                Expression::identifier("max"),
                 None,
                 Block::default()
             ),
             empty_with_step => NumericForStatement::new(
                 "i",
-                Expression::Identifier("start".to_owned()),
-                Expression::Identifier("max".to_owned()),
-                Some(Expression::Identifier("step".to_owned())),
+                Expression::identifier("start"),
+                Expression::identifier("max"),
+                Some(Expression::identifier("step")),
                 Block::default()
             ),
         ));
@@ -414,37 +491,36 @@ mod $mod_name {
         snapshot_node!($mod_name, $generator, repeat, write_statement => (
             empty => RepeatStatement::new(
                 Block::default(),
-                Expression::False
+                false
             ),
         ));
 
         snapshot_node!($mod_name, $generator, while_statement, write_statement => (
             empty => WhileStatement::new(
                 Block::default(),
-                Expression::False
+                false
             ),
         ));
 
         snapshot_node!($mod_name, $generator, last, write_last_statement => (
-            break_statement => LastStatement::Break,
-            continue_statement => LastStatement::Continue,
-            return_without_values => LastStatement::Return(Vec::new()),
-            return_one_expression => LastStatement::Return(vec![Expression::True]),
-            return_two_expressions => LastStatement::Return(vec![
-                Expression::True, Expression::Nil
-            ]),
+            break_statement => LastStatement::new_break(),
+            continue_statement => LastStatement::new_continue(),
+            return_without_values => ReturnStatement::default(),
+            return_one_expression => ReturnStatement::one(Expression::from(true)),
+            return_two_expressions => ReturnStatement::one(Expression::from(true))
+                .with_expression(Expression::nil()),
         ));
 
         snapshot_node!($mod_name, $generator, binary, write_expression => (
             true_and_false => BinaryExpression::new(
                 BinaryOperator::And,
-                Expression::True,
-                Expression::False
+                Expression::from(true),
+                Expression::from(false)
             ),
             true_equal_false =>BinaryExpression::new(
                 BinaryOperator::Equal,
-                Expression::True,
-                Expression::False
+                Expression::from(true),
+                Expression::from(false)
             ),
         ));
 
@@ -476,7 +552,7 @@ mod $mod_name {
 
         snapshot_node!($mod_name, $generator, prefix, write_prefix => (
             identifier => Prefix::from_name("foo"),
-            identifier_in_parenthese => Prefix::Parenthese(Prefix::from_name("foo").into()),
+            identifier_in_parenthese => Prefix::Parenthese(ParentheseExpression::new(Expression::identifier("foo"))),
         ));
 
         snapshot_node!($mod_name, $generator, string, write_expression => (
@@ -505,43 +581,43 @@ mod $mod_name {
         snapshot_node!($mod_name, $generator, table, write_expression => (
             empty => TableExpression::default(),
             list_with_single_value => TableExpression::new(vec![
-                TableEntry::Value(Expression::True),
+                TableEntry::Value(Expression::from(true)),
             ]),
             list_with_two_values => TableExpression::new(vec![
-                TableEntry::Value(Expression::True),
-                TableEntry::Value(Expression::False),
+                TableEntry::Value(Expression::from(true)),
+                TableEntry::Value(Expression::from(false)),
             ]),
             with_field_entry => TableExpression::new(vec![
-                TableEntry::Field("field".to_owned(), Expression::True),
+                TableFieldEntry::new("field", true).into(),
             ]),
             with_index_entry => TableExpression::new(vec![
-                TableEntry::Index(Expression::False, Expression::True),
+                TableIndexEntry::new(false, true).into(),
             ]),
             mixed_table => TableExpression::new(vec![
-                TableEntry::Value(Expression::True),
-                TableEntry::Field("field".to_owned(), Expression::True),
-                TableEntry::Index(Expression::False, Expression::True),
+                TableEntry::Value(Expression::from(true)),
+                TableFieldEntry::new("field", true).into(),
+                TableIndexEntry::new(false, true).into(),
             ]),
         ));
 
         snapshot_node!($mod_name, $generator, unary, write_expression => (
             not_true => UnaryExpression::new(
                 UnaryOperator::Not,
-                Expression::True,
+                true,
             ),
             two_unary_minus_breaks_between_them => UnaryExpression::new(
                 UnaryOperator::Minus,
                 UnaryExpression::new(
                     UnaryOperator::Minus,
-                    Expression::Identifier("a".to_owned()),
+                    Expression::identifier("a"),
                 ),
             ),
             wraps_in_parens_if_an_inner_binary_has_lower_precedence => UnaryExpression::new(
                 UnaryOperator::Not,
                 BinaryExpression::new(
                     BinaryOperator::Or,
-                    Expression::False,
-                    Expression::True,
+                    false,
+                    true,
                 ),
             ),
             does_not_wrap_in_parens_if_an_inner_binary_has_higher_precedence => UnaryExpression::new(
@@ -555,9 +631,9 @@ mod $mod_name {
         ));
 
         snapshot_node!($mod_name, $generator, arguments, write_arguments => (
-            empty_tuple => Arguments::Tuple(Vec::new()),
-            tuple_with_one_value => Arguments::Tuple(vec![Expression::True]),
-            tuple_with_two_values => Arguments::Tuple(vec![Expression::True, Expression::False]),
+            empty_tuple => TupleArguments::default(),
+            tuple_with_one_value => TupleArguments::new(vec![true.into()]),
+            tuple_with_two_values => TupleArguments::new(vec![true.into(), false.into()]),
         ));
     }
 }
@@ -567,4 +643,5 @@ mod $mod_name {
 
     snapshot_generator!(dense, DenseLuaGenerator::default());
     snapshot_generator!(readable, ReadableLuaGenerator::default());
+    snapshot_generator!(token_based, TokenBasedLuaGenerator::new(""));
 }
