@@ -48,7 +48,12 @@ impl<'a> TokenBasedLuaGenerator<'a> {
         }
     }
 
+    #[inline]
     fn write_token(&mut self, token: &Token) {
+        self.write_token_options(token, true)
+    }
+
+    fn write_token_options(&mut self, token: &Token, space_check: bool) {
         for trivia in token.iter_leading_trivia() {
             self.write_trivia(trivia);
         }
@@ -57,10 +62,6 @@ impl<'a> TokenBasedLuaGenerator<'a> {
 
         if self.currently_commenting {
             self.uncomment();
-        } else if let Some(next_character) = content.chars().next() {
-            if self.needs_space(next_character) {
-                self.output.push(' ');
-            }
         }
 
         if let Some(line_number) = token.get_line_number() {
@@ -70,6 +71,14 @@ impl<'a> TokenBasedLuaGenerator<'a> {
             }
         }
 
+        if space_check {
+            if let Some(next_character) = content.chars().next() {
+                if self.needs_space(next_character) {
+                    self.output.push(' ');
+                }
+            }
+        }
+
         self.push_str(content);
 
         for trivia in token.iter_trailing_trivia() {
@@ -78,7 +87,7 @@ impl<'a> TokenBasedLuaGenerator<'a> {
     }
 
     fn write_block_with_tokens(&mut self, block: &Block, tokens: &BlockTokens) {
-        let mut iterator = block.get_statements().iter().enumerate().peekable();
+        let mut iterator = block.iter_statements().enumerate().peekable();
 
         while let Some((index, statement)) = iterator.next() {
             self.write_statement(statement);
@@ -231,7 +240,7 @@ impl<'a> TokenBasedLuaGenerator<'a> {
 
     fn write_field_with_token(&mut self, field: &FieldExpression, token: &Token) {
         self.write_prefix(field.get_prefix());
-        self.write_token(token);
+        self.write_token_options(token, false);
         self.write_identifier(field.get_field());
     }
 
@@ -1392,6 +1401,9 @@ mod test {
         return_empty_function_with_two_arguments => "return function(a, b)\nend",
         return_empty_variadic_function_with_two_arguments => "return function(a, b, ...)\nend",
         return_identity_function => "return function(...)\n\treturn ...\nend",
+        return_field => "return math.huge",
+        return_field_ending_with_number => "return UDim2.new",
+        return_field_split_on_lines => "return value.\n\tproperty\n\t.name",
     );
 
     #[test]
@@ -1421,7 +1433,7 @@ mod test {
             .parse(code)
             .unwrap();
 
-        let call = match block.mutate_statements().last_mut().unwrap() {
+        let call = match block.iter_mut_statements().last().unwrap() {
             Statement::Call(call) => call,
             _ => panic!("unexpected statement"),
         };
