@@ -47,4 +47,18 @@ impl FieldExpression {
     pub fn mutate_prefix(&mut self) -> &mut Prefix {
         &mut self.prefix
     }
+
+    pub fn clear_comments(&mut self) {
+        self.field.clear_comments();
+        if let Some(token) = &mut self.token {
+            token.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        self.field.clear_whitespaces();
+        if let Some(token) = &mut self.token {
+            token.clear_whitespaces();
+        }
+    }
 }
