@@ -10,7 +10,35 @@ pub struct FunctionExpressionTokens {
     pub variable_arguments: Option<Token>,
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+impl FunctionExpressionTokens {
+    pub fn clear_comments(&mut self) {
+        self.function.clear_comments();
+        self.opening_parenthese.clear_comments();
+        self.closing_parenthese.clear_comments();
+        self.end.clear_comments();
+        self.parameter_commas
+            .iter_mut()
+            .for_each(Token::clear_comments);
+        if let Some(token) = &mut self.variable_arguments {
+            token.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        self.function.clear_whitespaces();
+        self.opening_parenthese.clear_whitespaces();
+        self.closing_parenthese.clear_whitespaces();
+        self.end.clear_whitespaces();
+        self.parameter_commas
+            .iter_mut()
+            .for_each(Token::clear_whitespaces);
+        if let Some(token) = &mut self.variable_arguments {
+            token.clear_whitespaces();
+        }
+    }
+}
+
+#[derive(Clone, Debug, Default, PartialEq, Eq)]
 pub struct FunctionExpression {
     block: Block,
     parameters: Vec<Identifier>,
@@ -105,15 +133,22 @@ impl FunctionExpression {
     pub fn has_parameters(&self) -> bool {
         !self.parameters.is_empty()
     }
-}
 
-impl Default for FunctionExpression {
-    fn default() -> Self {
-        Self {
-            block: Block::default(),
-            parameters: Vec::new(),
-            is_variadic: false,
-            tokens: None,
+    pub fn clear_comments(&mut self) {
+        self.parameters
+            .iter_mut()
+            .for_each(Identifier::clear_comments);
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        self.parameters
+            .iter_mut()
+            .for_each(Identifier::clear_whitespaces);
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_whitespaces();
         }
     }
 }
