@@ -6,6 +6,18 @@ pub struct IndexExpressionTokens {
     pub closing_bracket: Token,
 }
 
+impl IndexExpressionTokens {
+    pub fn clear_comments(&mut self) {
+        self.opening_bracket.clear_comments();
+        self.closing_bracket.clear_comments();
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        self.opening_bracket.clear_whitespaces();
+        self.closing_bracket.clear_whitespaces();
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct IndexExpression {
     prefix: Prefix,
@@ -56,4 +68,16 @@ impl IndexExpression {
     pub fn mutate_index(&mut self) -> &mut Expression {
         &mut self.index
     }
+
+    pub fn clear_comments(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_whitespaces();
+        }
+    }
 }
