@@ -68,6 +68,18 @@ impl DecimalNumber {
             self.float
         }
     }
+
+    pub fn clear_comments(&mut self) {
+        if let Some(token) = &mut self.token {
+            token.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        if let Some(token) = &mut self.token {
+            token.clear_whitespaces();
+        }
+    }
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -140,6 +152,18 @@ impl HexNumber {
             self.integer as f64
         }
     }
+
+    pub fn clear_comments(&mut self) {
+        if let Some(token) = &mut self.token {
+            token.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        if let Some(token) = &mut self.token {
+            token.clear_whitespaces();
+        }
+    }
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -190,6 +214,18 @@ impl BinaryNumber {
     pub fn get_raw_value(&self) -> u64 {
         self.value
     }
+
+    pub fn clear_comments(&mut self) {
+        if let Some(token) = &mut self.token {
+            token.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        if let Some(token) = &mut self.token {
+            token.clear_whitespaces();
+        }
+    }
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -242,6 +278,22 @@ impl NumberExpression {
             NumberExpression::Binary(number) => number.get_token(),
         }
     }
+
+    pub fn clear_comments(&mut self) {
+        match self {
+            NumberExpression::Decimal(number) => number.clear_comments(),
+            NumberExpression::Hex(number) => number.clear_comments(),
+            NumberExpression::Binary(number) => number.clear_comments(),
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        match self {
+            NumberExpression::Decimal(number) => number.clear_whitespaces(),
+            NumberExpression::Hex(number) => number.clear_whitespaces(),
+            NumberExpression::Binary(number) => number.clear_whitespaces(),
+        }
+    }
 }
 
 impl From<DecimalNumber> for NumberExpression {
