@@ -6,6 +6,18 @@ pub struct ParentheseTokens {
     pub right_parenthese: Token,
 }
 
+impl ParentheseTokens {
+    pub fn clear_comments(&mut self) {
+        self.left_parenthese.clear_comments();
+        self.right_parenthese.clear_comments();
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        self.left_parenthese.clear_whitespaces();
+        self.right_parenthese.clear_whitespaces();
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct ParentheseExpression {
     expression: Expression,
@@ -49,4 +61,16 @@ impl ParentheseExpression {
     pub fn get_tokens(&self) -> Option<&ParentheseTokens> {
         self.tokens.as_ref()
     }
+
+    pub fn clear_comments(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_whitespaces();
+        }
+    }
 }
