@@ -103,4 +103,16 @@ impl CompoundAssignStatement {
     pub fn mutate_value(&mut self) -> &mut Expression {
         &mut self.value
     }
+
+    pub fn clear_comments(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.operator.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.operator.clear_whitespaces();
+        }
+    }
 }
