@@ -6,6 +6,18 @@ pub struct RepeatTokens {
     pub until: Token,
 }
 
+impl RepeatTokens {
+    pub fn clear_comments(&mut self) {
+        self.repeat.clear_comments();
+        self.until.clear_comments();
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        self.repeat.clear_whitespaces();
+        self.until.clear_whitespaces();
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct RepeatStatement {
     block: Block,
@@ -56,4 +68,16 @@ impl RepeatStatement {
     pub fn get_tokens(&self) -> Option<&RepeatTokens> {
         self.tokens.as_ref()
     }
+
+    pub fn clear_comments(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_whitespaces();
+        }
+    }
 }
