@@ -7,6 +7,20 @@ pub struct WhileTokens {
     pub end: Token,
 }
 
+impl WhileTokens {
+    pub fn clear_comments(&mut self) {
+        self.r#while.clear_comments();
+        self.r#do.clear_comments();
+        self.end.clear_comments();
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        self.r#while.clear_whitespaces();
+        self.r#do.clear_whitespaces();
+        self.end.clear_whitespaces();
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct WhileStatement {
     block: Block,
@@ -57,4 +71,16 @@ impl WhileStatement {
     pub fn get_tokens(&self) -> Option<&WhileTokens> {
         self.tokens.as_ref()
     }
+
+    pub fn clear_comments(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_comments();
+        }
+    }
+
+    pub fn clear_whitespaces(&mut self) {
+        if let Some(tokens) = &mut self.tokens {
+            tokens.clear_whitespaces();
+        }
+    }
 }
