@@ -5,24 +5,21 @@ pub use lua_value::*;
 use crate::nodes::*;
 
 /// A struct to convert an Expression node into a LuaValue object.
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, Default, PartialEq, Eq)]
 pub struct Evaluator {
-    /// When evaluating expressions related to tables, this value tells the evaluator if
-    /// metamethods can have side effects. For example, indexing a normal table in Lua does not
-    /// have any side effects, but if the table is a metatable, it's __index metamethod can
-    /// possibly have side effects (since it can be a function call).
     pure_metamethods: bool,
 }
 
-impl Default for Evaluator {
-    fn default() -> Self {
-        Self {
-            pure_metamethods: false,
-        }
-    }
-}
-
 impl Evaluator {
+    /// When evaluating expressions related to tables, this value tells the evaluator if
+    /// metamethods can have side effects. For example, indexing a normal table in Lua does not
+    /// have any side effects, but if the table is a metatable, it's `__index` metamethod can
+    /// possibly have side effects (since it can be a function call).
+    pub fn assume_pure_metamethods(mut self) -> Self {
+        self.pure_metamethods = true;
+        self
+    }
+
     pub fn evaluate(&self, expression: &Expression) -> LuaValue {
         match expression {
             Expression::False(_) => LuaValue::False,
@@ -34,7 +31,16 @@ impl Evaluator {
             Expression::True(_) => LuaValue::True,
             Expression::Binary(binary) => self.evaluate_binary(binary),
             Expression::Unary(unary) => self.evaluate_unary(unary),
-            _ => LuaValue::Unknown,
+            Expression::Parenthese(parenthese) => {
+                // when the evaluator will be able to manage tuples, keep only the first element
+                // of the tuple here (or coerce the tuple to `nil` if it is empty)
+                self.evaluate(parenthese.inner_expression())
+            }
+            Expression::Call(_)
+            | Expression::Field(_)
+            | Expression::Identifier(_)
+            | Expression::Index(_)
+            | Expression::VariableArguments(_) => LuaValue::Unknown,
         }
     }
 
@@ -49,25 +55,46 @@ impl Evaluator {
             | Expression::True(_)
             | Expression::VariableArguments(_) => false,
             Expression::Binary(binary) => {
-                if self.pure_metamethods {
-                    self.has_side_effects(binary.left()) || self.has_side_effects(binary.right())
-                } else {
-                    let left = binary.left();
-                    let right = binary.right();
+                let left = binary.left();
+                let right = binary.right();
 
-                    self.maybe_table(&self.evaluate(left))
-                        || self.maybe_table(&self.evaluate(right))
-                        || self.has_side_effects(left)
-                        || self.has_side_effects(right)
+                let left_value = self.evaluate(left);
+                let left_side_effect = self.has_side_effects(binary.left());
+
+                match binary.operator() {
+                    BinaryOperator::And => {
+                        if left_value.is_truthy().unwrap_or(true) {
+                            left_side_effect || self.has_side_effects(binary.right())
+                        } else {
+                            left_side_effect
+                        }
+                    }
+                    BinaryOperator::Or => {
+                        if left_value.is_truthy().unwrap_or(false) {
+                            left_side_effect
+                        } else {
+                            left_side_effect || self.has_side_effects(binary.right())
+                        }
+                    }
+                    _ => {
+                        if self.pure_metamethods {
+                            left_side_effect || self.has_side_effects(binary.right())
+                        } else {
+                            self.maybe_metatable(&left_value)
+                                || self.maybe_metatable(&self.evaluate(right))
+                                || self.has_side_effects(left)
+                                || self.has_side_effects(right)
+                        }
+                    }
                 }
             }
             Expression::Unary(unary) => {
-                if self.pure_metamethods {
+                if self.pure_metamethods || matches!(unary.operator(), UnaryOperator::Not) {
                     self.has_side_effects(unary.get_expression())
                 } else {
                     let sub_expression = unary.get_expression();
 
-                    self.maybe_table(&self.evaluate(sub_expression))
+                    self.maybe_metatable(&self.evaluate(sub_expression))
                         || self.has_side_effects(sub_expression)
                 }
             }
@@ -125,15 +152,16 @@ impl Evaluator {
     }
 
     #[inline]
-    fn maybe_table(&self, value: &LuaValue) -> bool {
+    fn maybe_metatable(&self, value: &LuaValue) -> bool {
         match value {
             LuaValue::False
             | LuaValue::Function
             | LuaValue::Nil
             | LuaValue::Number(_)
             | LuaValue::String(_)
+            | LuaValue::Table
             | LuaValue::True => false,
-            LuaValue::Table | LuaValue::Unknown => true,
+            LuaValue::Unknown => true,
         }
     }
 
@@ -181,10 +209,12 @@ impl Evaluator {
                     _ => LuaValue::Unknown,
                 }
             }
-            BinaryOperator::LowerThan
-            | BinaryOperator::LowerOrEqualThan
-            | BinaryOperator::GreaterThan
-            | BinaryOperator::GreaterOrEqualThan => LuaValue::Unknown,
+            BinaryOperator::LowerThan => self.evaluate_relational(expression, |a, b| a < b),
+            BinaryOperator::LowerOrEqualThan => self.evaluate_relational(expression, |a, b| a <= b),
+            BinaryOperator::GreaterThan => self.evaluate_relational(expression, |a, b| a > b),
+            BinaryOperator::GreaterOrEqualThan => {
+                self.evaluate_relational(expression, |a, b| a >= b)
+            }
         }
     }
 
@@ -221,6 +251,51 @@ impl Evaluator {
         }
     }
 
+    fn evaluate_relational<F>(&self, expression: &BinaryExpression, operation: F) -> LuaValue
+    where
+        F: Fn(f64, f64) -> bool,
+    {
+        let left = self.evaluate(expression.left());
+
+        match left {
+            LuaValue::Number(left) => {
+                let right = self.evaluate(expression.right());
+
+                if let LuaValue::Number(right) = right {
+                    if operation(left, right) {
+                        LuaValue::True
+                    } else {
+                        LuaValue::False
+                    }
+                } else {
+                    LuaValue::Unknown
+                }
+            }
+            LuaValue::String(left) => {
+                let right = self.evaluate(expression.right());
+
+                if let LuaValue::String(right) = right {
+                    self.compare_strings(&left, &right, expression.operator())
+                } else {
+                    LuaValue::Unknown
+                }
+            }
+            _ => LuaValue::Unknown,
+        }
+    }
+
+    fn compare_strings(&self, left: &str, right: &str, operator: BinaryOperator) -> LuaValue {
+        LuaValue::from(match operator {
+            BinaryOperator::Equal => left == right,
+            BinaryOperator::NotEqual => left != right,
+            BinaryOperator::LowerThan => left < right,
+            BinaryOperator::LowerOrEqualThan => left <= right,
+            BinaryOperator::GreaterThan => left > right,
+            BinaryOperator::GreaterOrEqualThan => left >= right,
+            _ => return LuaValue::Unknown,
+        })
+    }
+
     fn evaluate_unary(&self, expression: &UnaryExpression) -> LuaValue {
         match expression.operator() {
             UnaryOperator::Not => self
@@ -258,8 +333,15 @@ mod test {
         true_expression(Expression::from(true)) => LuaValue::True,
         false_expression(Expression::from(false)) => LuaValue::False,
         nil_expression(Expression::nil()) => LuaValue::Nil,
-        number_expression(Expression::Number(DecimalNumber::new(0.0).into())) => LuaValue::Number(0.0),
+        number_expression(DecimalNumber::new(0.0)) => LuaValue::Number(0.0),
         string_expression(StringExpression::from_value("foo")) => LuaValue::String("foo".to_owned()),
+        true_wrapped_in_parens(ParentheseExpression::new(true)) => LuaValue::True,
+        false_wrapped_in_parens(ParentheseExpression::new(false)) => LuaValue::False,
+        nil_wrapped_in_parens(ParentheseExpression::new(Expression::nil())) => LuaValue::Nil,
+        number_wrapped_in_parens(ParentheseExpression::new(DecimalNumber::new(0.0)))
+            => LuaValue::Number(0.0),
+        string_wrapped_in_parens(ParentheseExpression::new(StringExpression::from_value("foo")))
+            => LuaValue::String("foo".to_owned()),
         table_expression(TableExpression::default()) => LuaValue::Table
     );
 
@@ -469,10 +551,30 @@ mod test {
                 StringExpression::empty(),
                 StringExpression::empty()
             ) => LuaValue::from(""),
+            number_lower_than_string(
+                BinaryOperator::LowerThan,
+                1.0,
+                StringExpression::empty()
+            ) => LuaValue::Unknown,
+            number_string_greater_than_number(
+                BinaryOperator::GreaterThan,
+                StringExpression::from_value("100"),
+                1.0
+            ) => LuaValue::Unknown,
+            number_string_greater_or_equal_than_number(
+                BinaryOperator::GreaterOrEqualThan,
+                StringExpression::from_value("100"),
+                100.0
+            ) => LuaValue::Unknown,
+            number_lower_or_equal_than_number_string(
+                BinaryOperator::GreaterOrEqualThan,
+                100.0,
+                StringExpression::from_value("100")
+            ) => LuaValue::Unknown,
         );
 
         macro_rules! evaluate_equality {
-            ($($name:ident ($left:expr, $right:expr) => $value:expr),*) => {
+            ($($name:ident ($left:expr, $right:expr) => $value:expr),* $(,)?) => {
                 $(
                     mod $name {
                         use super::*;
@@ -545,7 +647,160 @@ mod test {
             different_strings(
                 StringExpression::from_value("foo"),
                 StringExpression::from_value("bar")
-            ) => LuaValue::False
+            ) => LuaValue::False,
+        );
+
+        macro_rules! evaluate_equality_with_relational_operators {
+            ($($name:ident => $value:expr),* $(,)?) => {
+                $(
+                    mod $name {
+                        use super::*;
+
+                        #[test]
+                        fn lower() {
+                            let value: Expression = $value.into();
+                            let binary = BinaryExpression::new(BinaryOperator::LowerThan, value.clone(), value);
+                            assert_eq!(LuaValue::False, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn lower_or_equal() {
+                            let value: Expression = $value.into();
+                            let binary = BinaryExpression::new(BinaryOperator::LowerOrEqualThan, value.clone(), value);
+                            assert_eq!(LuaValue::True, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn greater() {
+                            let value: Expression = $value.into();
+                            let binary = BinaryExpression::new(BinaryOperator::GreaterThan, value.clone(), value);
+                            assert_eq!(LuaValue::False, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn greater_or_equal() {
+                            let value: Expression = $value.into();
+                            let binary = BinaryExpression::new(BinaryOperator::GreaterOrEqualThan, value.clone(), value);
+                            assert_eq!(LuaValue::True, Evaluator::default().evaluate(&binary.into()));
+                        }
+                    }
+                )*
+            };
+        }
+
+        evaluate_equality_with_relational_operators!(
+            zero => 1.0,
+            one => 1.0,
+            hundred => 100.0,
+            string => StringExpression::from_value("var"),
+        );
+
+        macro_rules! evaluate_strict_relational_operators {
+            ($($name_lower:ident($lower:expr) < $name_greater:ident($greater:expr)),* $(,)?) => {
+                mod lower_or_greater_than {
+                    use super::*;
+                    paste::paste! {
+
+                    $(
+                        #[test]
+                        fn [<$name_lower _lower_than_ $name_greater>]() {
+                            let binary = BinaryExpression::new(
+                                BinaryOperator::LowerThan,
+                                $lower,
+                                $greater,
+                            );
+                            assert_eq!(LuaValue::True, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn [<$name_lower _lower_or_equal_than_ $name_greater>]() {
+                            let binary = BinaryExpression::new(
+                                BinaryOperator::LowerOrEqualThan,
+                                $lower,
+                                $greater,
+                            );
+                            assert_eq!(LuaValue::True, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn [<$name_lower _greater_than_ $name_greater>]() {
+                            let binary = BinaryExpression::new(
+                                BinaryOperator::GreaterThan,
+                                $lower,
+                                $greater,
+                            );
+                            assert_eq!(LuaValue::False, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn [<$name_lower _greater_or_equal_than_ $name_greater>]() {
+                            let binary = BinaryExpression::new(
+                                BinaryOperator::GreaterOrEqualThan,
+                                $lower,
+                                $greater,
+                            );
+                            assert_eq!(LuaValue::False, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn [<$name_greater _lower_than_ $name_lower>]() {
+                            let binary = BinaryExpression::new(
+                                BinaryOperator::LowerThan,
+                                $greater,
+                                $lower,
+                            );
+                            assert_eq!(LuaValue::False, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn [<$name_greater _lower_or_equal_than_ $name_lower>]() {
+                            let binary = BinaryExpression::new(
+                                BinaryOperator::LowerOrEqualThan,
+                                $greater,
+                                $lower,
+                            );
+                            assert_eq!(LuaValue::False, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn [<$name_greater _greater_than_ $name_lower>]() {
+                            let binary = BinaryExpression::new(
+                                BinaryOperator::GreaterThan,
+                                $greater,
+                                $lower,
+                            );
+                            assert_eq!(LuaValue::True, Evaluator::default().evaluate(&binary.into()));
+                        }
+
+                        #[test]
+                        fn [<$name_greater _greater_or_equal_than_ $name_lower>]() {
+                            let binary = BinaryExpression::new(
+                                BinaryOperator::GreaterOrEqualThan,
+                                $greater,
+                                $lower,
+                            );
+                            assert_eq!(LuaValue::True, Evaluator::default().evaluate(&binary.into()));
+                        }
+                    )*
+
+                    }
+                }
+            };
+        }
+
+        evaluate_strict_relational_operators!(
+            one(1.0) < hundred(100.0),
+            minus_15(-15.0) < minus_2_5(-2.5),
+            string_a(StringExpression::from_value("a"))
+                < string_b(StringExpression::from_value("b")),
+            string_a(StringExpression::from_value("a"))
+                < string_aa(StringExpression::from_value("aa")),
+            string_1(StringExpression::from_value("1"))
+                < string_a(StringExpression::from_value("a")),
+            string_111(StringExpression::from_value("111"))
+                < string_a(StringExpression::from_value("a")),
+            empty_string(StringExpression::from_value(""))
+                < string_colon(StringExpression::from_value(":")),
         );
     }
 
@@ -611,15 +866,80 @@ mod test {
             Expression::from(true),
             FunctionCall::from_name("foo"),
         ),
+        binary_false_or_call => BinaryExpression::new(
+            BinaryOperator::Or,
+            Expression::from(false),
+            FunctionCall::from_name("var"),
+        ),
+        addition_unknown_variable_and_number => BinaryExpression::new(
+            BinaryOperator::Plus,
+            Expression::identifier("var"),
+            1.0,
+        ),
+        addition_number_with_unknown_variable => BinaryExpression::new(
+            BinaryOperator::Plus,
+            1.0,
+            Expression::identifier("var"),
+        ),
+        unary_minus_on_variable => UnaryExpression::new(UnaryOperator::Minus, Identifier::new("var")),
+        length_on_variable => UnaryExpression::new(UnaryOperator::Length, Identifier::new("var")),
+        field_index => FieldExpression::new(Identifier::new("var"), "field"),
+        table_value_with_call_in_entry => TableExpression::default()
+            .append_array_value(FunctionCall::from_name("call")),
     );
 
     has_no_side_effects!(
         true_value => Expression::from(true),
         false_value => Expression::from(false),
         nil_value => Expression::nil(),
+        table_value => TableExpression::default(),
         number_value => Expression::Number(DecimalNumber::new(0.0).into()),
         string_value => StringExpression::from_value(""),
         identifier => Expression::identifier("foo"),
         identifier_in_parentheses => Expression::identifier("foo").in_parentheses(),
+        binary_false_and_call => BinaryExpression::new(
+            BinaryOperator::And,
+            Expression::from(false),
+            FunctionCall::from_name("foo"),
+        ),
+        binary_true_or_call => BinaryExpression::new(
+            BinaryOperator::Or,
+            Expression::from(true),
+            FunctionCall::from_name("foo"),
+        ),
+        not_variable => UnaryExpression::new(UnaryOperator::Not, Identifier::new("var")),
     );
+
+    mod assume_pure_metamethods {
+        use super::*;
+
+        macro_rules! has_no_side_effects {
+            ($($name:ident => $expression:expr),* $(,)?) => {
+                $(
+                    #[test]
+                    fn $name() {
+                        let evaluator = Evaluator::default().assume_pure_metamethods();
+                        assert!(!evaluator.has_side_effects(&$expression.into()));
+                    }
+                )*
+            };
+        }
+
+        has_no_side_effects!(
+            addition_unknown_variable_and_number => BinaryExpression::new(
+                BinaryOperator::Plus,
+                Expression::identifier("foo"),
+                1.0,
+            ),
+            addition_number_with_unknown_variable => BinaryExpression::new(
+                BinaryOperator::Plus,
+                1.0,
+                Expression::identifier("foo"),
+            ),
+            unary_minus_on_variable => UnaryExpression::new(UnaryOperator::Minus, Identifier::new("var")),
+            length_on_variable => UnaryExpression::new(UnaryOperator::Length, Identifier::new("var")),
+            not_on_variable => UnaryExpression::new(UnaryOperator::Not, Identifier::new("var")),
+            field_index => FieldExpression::new(Identifier::new("var"), "field"),
+        );
+    }
 }
