@@ -2,23 +2,33 @@
 
 mod call_parens;
 mod compute_expression;
+mod configuration_error;
+mod convert_index_to_field;
 mod empty_do;
 mod group_local;
 mod inject_value;
 mod method_def;
 mod no_local_function;
+mod remove_comments;
+mod remove_spaces;
 mod rename_variables;
+mod rule_property;
 mod unused_if_branch;
 mod unused_while;
 
 pub use call_parens::*;
 pub use compute_expression::*;
+pub use configuration_error::RuleConfigurationError;
+pub use convert_index_to_field::*;
 pub use empty_do::*;
 pub use group_local::*;
 pub use inject_value::*;
 pub use method_def::*;
 pub use no_local_function::*;
+pub use remove_comments::*;
+pub use remove_spaces::*;
 pub use rename_variables::*;
+pub use rule_property::*;
 pub use unused_if_branch::*;
 pub use unused_while::*;
 
@@ -31,62 +41,6 @@ use std::collections::HashMap;
 use std::fmt;
 use std::str::FromStr;
 
-/// In order to be able to weakly-type the properties of any rule, this enum makes it possible to
-/// easily use serde to gather the value associated with a property.
-#[derive(Debug, Serialize, Deserialize)]
-#[serde(untagged)]
-pub enum RulePropertyValue {
-    Boolean(bool),
-    String(String),
-    Usize(usize),
-    Float(f64),
-    StringList(Vec<String>),
-    None,
-}
-
-/// When implementing the configure method of the Rule trait, the method returns a result that uses
-/// this error type.
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub enum RuleConfigurationError {
-    /// When a rule gets an unknown property. The string should be the unknown field name.
-    UnexpectedProperty(String),
-    /// When a rule has a required property. The string should be the field name.
-    MissingProperty(String),
-    /// When a property is associated with something else than an expected string. The string is
-    /// the property name.
-    StringExpected(String),
-    /// When a property is associated with something else than an expected unsigned number. The
-    /// string is the property name.
-    UsizeExpected(String),
-    /// When a property is associated with something else than an expected list of strings. The
-    /// string is the property name.
-    StringListExpected(String),
-    /// When the value type is invalid. The string is the property name that was given the wrong
-    /// value type.
-    UnexpectedValueType(String),
-}
-
-impl fmt::Display for RuleConfigurationError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        use RuleConfigurationError::*;
-
-        match self {
-            UnexpectedProperty(property) => write!(f, "unexpected field '{}'", property),
-            MissingProperty(property) => write!(f, "missing required field '{}'", property),
-            StringExpected(property) => write!(f, "string value expected for field '{}'", property),
-            UsizeExpected(property) => {
-                write!(f, "unsigned integer expected for field '{}'", property)
-            }
-            StringListExpected(property) => {
-                write!(f, "list of string expected for field '{}'", property)
-            }
-            UnexpectedValueType(property) => write!(f, "unexpected type for field '{}'", property),
-        }
-    }
-}
-
-pub type RuleProperties = HashMap<String, RulePropertyValue>;
-
 /// The intent of this struct is to hold data shared across all rules applied to a file.
 #[derive(Debug, Clone, Default)]
 pub struct Context {}
@@ -104,11 +58,15 @@ pub trait RuleConfiguration {
     /// The rule deserializer will construct the default rule and then send the properties through
     /// this method to modify the behavior of the rule.
     fn configure(&mut self, properties: RuleProperties) -> Result<(), RuleConfigurationError>;
-    /// This method should the unique name of the rule.
+    /// This method should return the unique name of the rule.
     fn get_name(&self) -> &'static str;
     /// For implementing the serialize trait on the Rule trait, this method should return all
     /// properties that differs from their default value.
     fn serialize_to_properties(&self) -> RuleProperties;
+    /// Returns `true` if the rule has at least one property.
+    fn has_properties(&self) -> bool {
+        !self.serialize_to_properties().is_empty()
+    }
 }
 
 pub trait FlawlessRule {
@@ -127,13 +85,14 @@ impl<T: FlawlessRule + RuleConfiguration> Rule for T {
 /// processed block will work as much as the original one.
 pub fn get_default_rules() -> Vec<Box<dyn Rule>> {
     vec![
+        Box::new(RemoveSpaces::default()),
+        Box::new(RemoveComments::default()),
         Box::new(ComputeExpression::default()),
         Box::new(RemoveUnusedIfBranch::default()),
         Box::new(RemoveUnusedWhile::default()),
         Box::new(RemoveEmptyDo::default()),
         Box::new(RemoveMethodDefinition::default()),
-        Box::new(ConvertLocalFunctionToAssign::default()),
-        Box::new(GroupLocalAssignment::default()),
+        Box::new(ConvertIndexToField::default()),
         Box::new(RenameVariables::default()),
         Box::new(RemoveFunctionCallParens::default()),
     ]
@@ -145,14 +104,17 @@ impl FromStr for Box<dyn Rule> {
     fn from_str(string: &str) -> Result<Self, Self::Err> {
         let rule: Box<dyn Rule> = match string {
             COMPUTE_EXPRESSIONS_RULE_NAME => Box::new(ComputeExpression::default()),
+            CONVERT_INDEX_TO_FIELD_RULE_NAME => Box::new(ConvertIndexToField::default()),
             CONVERT_LOCAL_FUNCTION_TO_ASSIGN_RULE_NAME => {
                 Box::new(ConvertLocalFunctionToAssign::default())
             }
             GROUP_LOCAL_ASSIGNMENT => Box::new(GroupLocalAssignment::default()),
             INJECT_GLOBAL_VALUE_RULE_NAME => Box::new(InjectGlobalValue::default()),
+            REMOVE_COMMENTS_RULE_NAME => Box::new(RemoveComments::default()),
             REMOVE_EMPTY_DO_RULE_NAME => Box::new(RemoveEmptyDo::default()),
             REMOVE_FUNCTION_CALL_PARENS => Box::new(RemoveFunctionCallParens::default()),
             REMOVE_METHOD_DEFINITION_RULE_NAME => Box::new(RemoveMethodDefinition::default()),
+            REMOVE_SPACES_RULE_NAME => Box::new(RemoveSpaces::default()),
             REMOVE_UNUSED_IF_BRANCH_RULE_NAME => Box::new(RemoveUnusedIfBranch::default()),
             REMOVE_UNUSED_WHILE_RULE_NAME => Box::new(RemoveUnusedWhile::default()),
             RENAME_VARIABLES_RULE_NAME => Box::new(RenameVariables::default()),
@@ -265,6 +227,37 @@ fn verify_no_rule_properties(properties: &RuleProperties) -> Result<(), RuleConf
     Ok(())
 }
 
+fn verify_required_properties(
+    properties: &RuleProperties,
+    names: &[&str],
+) -> Result<(), RuleConfigurationError> {
+    for name in names.iter() {
+        if !properties.contains_key(*name) {
+            return Err(RuleConfigurationError::MissingProperty(name.to_string()));
+        }
+    }
+    Ok(())
+}
+
+fn verify_property_collisions(
+    properties: &RuleProperties,
+    names: &[&str],
+) -> Result<(), RuleConfigurationError> {
+    let mut exists = false;
+    for name in names.iter() {
+        if properties.contains_key(*name) {
+            if exists {
+                return Err(RuleConfigurationError::PropertyCollision(
+                    names.iter().map(ToString::to_string).collect(),
+                ));
+            } else {
+                exists = true;
+            }
+        }
+    }
+    Ok(())
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
